@@ -426,6 +426,8 @@ ags_composite_toolbar_init(AgsCompositeToolbar *composite_toolbar)
   composite_toolbar->wave_time_stretch_buffer = (GtkDialog *) ags_time_stretch_buffer_dialog_new();
 
   composite_toolbar->program_ramp_marker = (GtkDialog *) ags_ramp_marker_dialog_new();
+
+  composite_toolbar->snap_to_zoom = NULL;
 }
 
 AgsUUID*
@@ -1514,6 +1516,26 @@ ags_composite_toolbar_set_option(AgsCompositeToolbar *composite_toolbar, guint o
   }else if(composite_toolbar->beats_type != NULL){
     sibling = gtk_widget_get_parent((GtkWidget *) composite_toolbar->beats_type);
   }
+
+  /* snap_to_zoom */
+  if((AGS_COMPOSITE_TOOLBAR_HAS_SNAP_TO_ZOOM & option) != 0 &&
+     composite_toolbar->snap_to_zoom == NULL){
+    GtkBox *box;
+
+    box = (GtkBox *) gtk_box_new(GTK_ORIENTATION_HORIZONTAL,
+				 0);
+    gtk_box_insert_child_after((GtkBox *) composite_toolbar,
+			       (GtkWidget *) box,
+			       sibling);
+
+    composite_toolbar->snap_to_zoom = (GtkSpinButton *) gtk_check_button_new_with_label(i18n("snap to zoom"));
+    gtk_box_append(box,
+		   (GtkWidget *) composite_toolbar->snap_to_zoom);
+
+    sibling = (GtkWidget *) box;
+  }else if(composite_toolbar->snap_to_zoom != NULL){
+    sibling = gtk_widget_get_parent((GtkWidget *) composite_toolbar->snap_to_zoom);
+  }
   
   /* set option */
   composite_toolbar->option |= option;
@@ -1575,6 +1597,14 @@ ags_composite_toolbar_unset_option(AgsCompositeToolbar *composite_toolbar, guint
 
     composite_toolbar->beats_type = NULL;
   }
+
+  if((AGS_COMPOSITE_TOOLBAR_HAS_SNAP_TO_ZOOM & option) != 0 &&
+     composite_toolbar->snap_to_zoom != NULL){
+    gtk_box_remove((GtkBox *) composite_toolbar,
+		   gtk_widget_get_parent((GtkWidget *) composite_toolbar->snap_to_zoom));
+    
+    composite_toolbar->snap_to_zoom = NULL;
+  }
   
   /* unset option */
   composite_toolbar->option &= (~option);
@@ -2284,7 +2314,8 @@ ags_composite_toolbar_scope_create_and_connect(AgsCompositeToolbar *composite_to
 				      AGS_COMPOSITE_TOOLBAR_HAS_OPACITY |
 				      AGS_COMPOSITE_TOOLBAR_HAS_BEATS |
 				      AGS_COMPOSITE_TOOLBAR_HAS_BEATS_TYPE |
-				      AGS_COMPOSITE_TOOLBAR_HAS_PORT));
+				      AGS_COMPOSITE_TOOLBAR_HAS_PORT |
+				      AGS_COMPOSITE_TOOLBAR_HAS_SNAP_TO_ZOOM));
 
   composite_toolbar->selected_tool = NULL;
   
@@ -2354,7 +2385,8 @@ ags_composite_toolbar_scope_create_and_connect(AgsCompositeToolbar *composite_to
       ags_composite_toolbar_set_option(composite_toolbar,
 				       (AGS_COMPOSITE_TOOLBAR_HAS_MENU_TOOL |
 					AGS_COMPOSITE_TOOLBAR_HAS_ZOOM |
-					AGS_COMPOSITE_TOOLBAR_HAS_OPACITY));
+					AGS_COMPOSITE_TOOLBAR_HAS_OPACITY |
+					AGS_COMPOSITE_TOOLBAR_HAS_SNAP_TO_ZOOM));
 
       /* connect */      
       ags_connectable_connect_connection(AGS_CONNECTABLE(composite_toolbar),
@@ -2374,9 +2406,15 @@ ags_composite_toolbar_scope_create_and_connect(AgsCompositeToolbar *composite_to
 
       ags_connectable_connect_connection(AGS_CONNECTABLE(composite_toolbar),
 					 (GObject *) composite_toolbar->opacity);
+
+      ags_connectable_connect_connection(AGS_CONNECTABLE(composite_toolbar),
+					 (GObject *) composite_toolbar->snap_to_zoom);
       
       gtk_toggle_button_set_active(composite_toolbar->position,
 				   TRUE);
+
+      gtk_check_button_set_active(composite_toolbar->snap_to_zoom,
+				  TRUE);
       
       success = TRUE;
     }else if(!g_strcmp0(scope,
